@@ -8,7 +8,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; version 2.1 of
- * the License.
+ * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,17 +24,14 @@
 
 /**
  * SECTION:hildon-find-toolbar
- * @short_description: A special toolbar to be used with HildonWindow.
+ * @short_description: A toolbar with a search field.
  * @see_also: #HildonWindow
  *
- * HildonFindToolbar is a predefined toolbar for text searching purpose. 
- * It contains a GtkListStore which has the text items that the user has 
- * searched. But once the application is terminated, or HildonFindToolbar 
- * is trashed. Programmer is responsible for getting the GtkListStore through 
- * property "list", if he/she wants to use the information in the future.
- * And through the same property, programmer is able to set the GtkListStore. 
- * Note, once the search button is pressed, string in the GtkComboxEntry is 
- * automatically added to the existing model, unless it is empty.
+ * HildonFindToolbar is a toolbar that contains a search entry and a dropdown
+ * list with previously searched strings. The list is represented using a 
+ * #GtkListStore and can be accesed using a property 'list'. Entries are added
+ * automatically to the list when the search button is pressed.
+ *
  */    
 
 #ifdef                                          HAVE_CONFIG_H
@@ -111,10 +108,12 @@ static void
 hildon_find_toolbar_emit_close                  (GtkButton *button, 
                                                  gpointer self);
 
+#ifdef MAEMO_GTK 
 static void
 hildon_find_toolbar_emit_invalid_input          (GtkEntry *entry, 
                                                  GtkInvalidInputType type, 
                                                  gpointer self);
+#endif
 
 static void
 hildon_find_toolbar_entry_activate              (GtkWidget *widget,
@@ -484,6 +483,7 @@ static void
 hildon_find_toolbar_emit_close                  (GtkButton *button, 
                                                  gpointer self)
 {
+#ifdef MAEMO_GTK 
     HildonFindToolbarPrivate *priv = HILDON_FIND_TOOLBAR_GET_PRIVATE (self);
     g_assert (priv);
 
@@ -492,11 +492,13 @@ hildon_find_toolbar_emit_close                  (GtkButton *button,
     {
         hildon_gtk_im_context_hide (GTK_ENTRY (entry)->im_context);
     }
+#endif
 
     /* Clicked close button */
     g_signal_emit_by_name (self, "close", NULL);
 }
 
+#ifdef MAEMO_GTK 
 static void
 hildon_find_toolbar_emit_invalid_input          (GtkEntry *entry, 
                                                  GtkInvalidInputType type, 
@@ -505,6 +507,7 @@ hildon_find_toolbar_emit_invalid_input          (GtkEntry *entry,
     if(type == GTK_INVALID_INPUT_MAX_CHARS_REACHED)
         g_signal_emit_by_name (self, "invalid_input", NULL);
 }
+#endif
 
 static void
 hildon_find_toolbar_entry_activate              (GtkWidget *widget,
@@ -650,7 +653,7 @@ hildon_find_toolbar_class_init                  (HildonFindToolbarClass *klass)
      * 
      * Gets emitted when the maximum search prefix length is reached and
      * user tries to type more.
-     */ 
+     */
     HildonFindToolbar_signal[INVALID_INPUT] = 
         g_signal_new(
                 "invalid_input", HILDON_TYPE_FIND_TOOLBAR,
@@ -699,9 +702,12 @@ hildon_find_toolbar_init                        (HildonFindToolbar *self)
 
     /* ComboBoxEntry for search prefix string / history list */
     priv->entry_combo_box = GTK_COMBO_BOX_ENTRY (gtk_combo_box_entry_new ());
+
+#ifdef MAEMO_GTK
     g_signal_connect (hildon_find_toolbar_get_entry(priv),
             "invalid_input", 
             G_CALLBACK(hildon_find_toolbar_emit_invalid_input), self);
+#endif
 
     entry_combo_box_container = gtk_tool_item_new ();
 
