@@ -8,7 +8,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; version 2.1 of
- * the License.
+ * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +24,7 @@
 
 /**
  * SECTION:hildon-set-password-dialog
- * @short_description: A dialog used to set, change or remove a password
+ * @short_description: A dialog used to set, change or remove a password.
  * @see_also: #HildonGetPasswordDialog
  *
  * HildonSetPasswordDialog allows setting and changing a password. 
@@ -275,7 +275,11 @@ create_contents                                 (HildonSetPasswordDialog *dialog
 
     /* Create the password field */
     priv->pwd1st_entry = gtk_entry_new ();
+
+#ifdef MAEMO_GTK
     g_object_set (priv->pwd1st_entry, "hildon-input-mode", HILDON_GTK_INPUT_MODE_FULL, NULL);
+#endif
+
     gtk_entry_set_visibility (GTK_ENTRY(priv->pwd1st_entry), FALSE);
     gtk_widget_show (priv->pwd1st_entry);
     priv->pwd1st_caption = hildon_caption_new (group,
@@ -291,7 +295,11 @@ create_contents                                 (HildonSetPasswordDialog *dialog
 
     /* Create the password verify field */
     priv->pwd2nd_entry = gtk_entry_new();
+
+#ifdef MAEMO_GTK
     g_object_set (priv->pwd2nd_entry, "hildon-input-mode", HILDON_GTK_INPUT_MODE_FULL, NULL);
+#endif
+
     gtk_widget_show (priv->pwd2nd_entry);
     priv->pwd2nd_caption = hildon_caption_new (group,
             priv->pwd2nd_caption_string,
