@@ -8,7 +8,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; version 2.1 of
- * the License.
+ * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -185,6 +185,9 @@ hildon_window_key_release_event                 (GtkWidget *widget,
 static gboolean
 hildon_window_window_state_event                (GtkWidget *widget, 
                                                  GdkEventWindowState *event);
+static gboolean
+hildon_window_focus_out_event                   (GtkWidget *widget, 
+                                                 GdkEventFocus *event);
 
 static void
 hildon_window_notify                            (GObject *gobject, 
@@ -194,7 +197,9 @@ static void
 hildon_window_is_topmost_notify                 (HildonWindow *window);
 
 static gboolean
-hildon_window_toggle_menu                       (HildonWindow * self);
+hildon_window_toggle_menu                       (HildonWindow * self,
+						 guint button,
+						 guint32 time);
 
 static gboolean
 hildon_window_escape_timeout                    (gpointer data);
@@ -288,6 +293,7 @@ hildon_window_class_init                        (HildonWindowClass * window_clas
     widget_class->key_press_event       = hildon_window_key_press_event;
     widget_class->key_release_event     = hildon_window_key_release_event;
     widget_class->window_state_event    = hildon_window_window_state_event;
+    widget_class->focus_out_event       = hildon_window_focus_out_event;
 
     /* now the object stuff */
     object_class->finalize              = hildon_window_finalize;
@@ -380,6 +386,11 @@ hildon_window_finalize                          (GObject * obj_self)
     
     self = HILDON_WINDOW (obj_self);
 
+    if (priv->escape_timeout) {
+      g_source_remove (priv->escape_timeout);
+      priv->escape_timeout = 0;
+    }
+
     g_free (priv->borders);
     g_free (priv->toolbar_borders);
 
@@ -757,7 +768,8 @@ hildon_window_destroy                           (GtkObject *obj)
 {
     HildonWindow *self = HILDON_WINDOW (obj);
     HildonWindowPrivate *priv = HILDON_WINDOW_GET_PRIVATE (obj);
-    GList *menu_list;
+    GList *menu_list = NULL;
+    GList *menu_node = NULL;
     
     g_assert (priv != NULL);
 
@@ -780,22 +792,31 @@ hildon_window_destroy                           (GtkObject *obj)
     }
 
     menu_list = g_list_copy (gtk_menu_get_for_attach_widget (GTK_WIDGET (obj)));
+    menu_node = menu_list;
 
-    while (menu_list)
+    while (menu_node)
     {
-        if (GTK_IS_MENU(menu_list->data))
+        if (GTK_IS_MENU (menu_node->data))
         {
-            if (GTK_WIDGET_VISIBLE (GTK_WIDGET (menu_list->data)))
+            if (GTK_WIDGET_VISIBLE (GTK_WIDGET (menu_node->data)))
             {
-                gtk_menu_popdown (GTK_MENU (menu_list->data));
-                gtk_menu_shell_deactivate (GTK_MENU_SHELL (menu_list->data));
+                gtk_menu_popdown (GTK_MENU (menu_node->data));
+                gtk_menu_shell_deactivate (GTK_MENU_SHELL (menu_node->data));
+            }
+            gtk_menu_detach (GTK_MENU (menu_node->data));
+
+            /* Destroy it, but only if it's not a common menu */
+            if (priv->program && 
+                hildon_program_get_common_menu (priv->program) != menu_node->data) {
+                    gtk_object_destroy (GTK_OBJECT (menu_node->data));
+                    g_object_unref (menu_node->data);
             }
-            gtk_menu_detach (GTK_MENU (menu_list->data));
         }
-        menu_list = menu_list->next;
+        menu_node = menu_node->next;
     }
 
     g_list_free (menu_list);
+    menu_list = NULL;
 
     if (priv->program)
     {
@@ -1002,6 +1023,7 @@ Window
 hildon_window_get_active_window                 (void)
 {
     Atom realtype;
+    gint xerror;
     int format;
     int status;
     Window ret;
@@ -1017,11 +1039,13 @@ hildon_window_get_active_window                 (void)
 
     win.win = NULL;
 
+    gdk_error_trap_push ();
     status = XGetWindowProperty (GDK_DISPLAY(), GDK_ROOT_WINDOW(),
             active_app_atom, 0L, 16L,
             0, XA_WINDOW, &realtype, &format,
             &n, &extra, &win.char_pointer);
-    if (!(status == Success && realtype == XA_WINDOW && format == 32
+    xerror = gdk_error_trap_pop ();
+    if (xerror || !(status == Success && realtype == XA_WINDOW && format == 32
                 && n == 1 && win.win != NULL))
     {
         if (win.win != NULL)
@@ -1061,7 +1085,7 @@ hildon_window_event_filter                      (GdkXEvent *xevent,
 
         if (xclient_message_type_check (cm, "_MB_GRAB_TRANSFER"))
         {
-            hildon_window_toggle_menu (HILDON_WINDOW ( data ));
+            hildon_window_toggle_menu (HILDON_WINDOW ( data ), cm->data.l[2], cm->data.l[0]);
             return GDK_FILTER_REMOVE;
         }
         /* opera hack clipboard client message */
@@ -1132,7 +1156,7 @@ hildon_window_key_press_event                   (GtkWidget *widget,
     switch (event->keyval)
     {
         case HILDON_HARDKEY_MENU:
-            if (hildon_window_toggle_menu (HILDON_WINDOW (widget)))
+            if (hildon_window_toggle_menu (HILDON_WINDOW (widget), 0, GDK_CURRENT_TIME))
                 return TRUE;
             break;
         case HILDON_HARDKEY_ESC:
@@ -1198,6 +1222,25 @@ hildon_window_window_state_event                (GtkWidget *widget,
     }
 }
 
+/*
+ * If the window lost focus while the user started to press the ESC key, we
+ * won't get the release event. We need to stop the timeout.
+ */
+static gboolean
+hildon_window_focus_out_event                   (GtkWidget *widget, 
+                                                 GdkEventFocus *event)
+{
+  HildonWindowPrivate *priv = HILDON_WINDOW_GET_PRIVATE (widget);
+
+  if (priv->escape_timeout)
+  {
+      g_source_remove (priv->escape_timeout);
+      priv->escape_timeout = 0;
+  }
+
+  return GTK_WIDGET_CLASS (parent_class)->focus_out_event (widget, event);
+}
+
 /*
  * The menu popuping needs a menu popup-function
  */
@@ -1254,18 +1297,6 @@ hildon_window_is_topmost_notify                 (HildonWindow *window)
     {
         hildon_window_take_common_toolbar (window);
     }
-
-    else
-    {
-        /* If the window lost focus while the user started to press
-         * the ESC key, we won't get the release event. We need to
-         * stop the timeout*/
-        if (priv->escape_timeout)
-        {
-            g_source_remove (priv->escape_timeout);
-            priv->escape_timeout = 0;
-        }
-    }
 }
 
 /*
@@ -1465,12 +1496,15 @@ hildon_window_update_title                      (HildonWindow *window)
     {
         const gchar *old_title = gtk_window_get_title (GTK_WINDOW (window));
 
-        if (old_title && old_title[0])
+        if (old_title)
         {
             gchar *title = NULL;
-
-            title = g_strjoin (TITLE_SEPARATOR, application_name,
-                    old_title, NULL);
+                
+            if (strlen (old_title) == 0) 
+                title = g_strdup (application_name);
+            else
+                title = g_strjoin (TITLE_SEPARATOR, application_name,
+                                   old_title, NULL);
 
             gdk_window_set_title (GTK_WIDGET (window)->window, title);
 
@@ -1493,7 +1527,9 @@ detach_menu_func                                (GtkWidget *attach_widget,
  * to toggle)
  */
 static gboolean
-hildon_window_toggle_menu                       (HildonWindow * self)
+hildon_window_toggle_menu                       (HildonWindow * self,
+						 guint button,
+						 guint32 time)
 {
     GtkMenu *menu_to_use = NULL;
     GList *menu_children = NULL;
@@ -1555,16 +1591,14 @@ hildon_window_toggle_menu                       (HildonWindow * self)
             gtk_menu_popup (menu_to_use, NULL, NULL,
                     (GtkMenuPositionFunc)
                     hildon_window_menu_popup_func_full,
-                    self, 0, 
-                    gtk_get_current_event_time ());
+                    self, button, time);
         }
         else
         {
             gtk_menu_popup (menu_to_use, NULL, NULL,
                     (GtkMenuPositionFunc)
                     hildon_window_menu_popup_func,
-                    self, 0, 
-                    gtk_get_current_event_time ());
+                    self, button, time);
         }
         gtk_menu_shell_select_first (GTK_MENU_SHELL (menu_to_use), TRUE);
         return TRUE;
@@ -1660,26 +1694,30 @@ hildon_window_add_with_scrollbar                (HildonWindow *self,
  * @self: A @HildonWindow
  * @toolbar: A #GtkToolbar to add to the HildonWindow
  *
- * Adds a toolbar to the window.
+ * Adds a toolbar to the window. Note that the toolbar is not automatically
+ * shown. You need to call #gtk_widget_show_all on it to make it visible. 
+ * It's also possible to hide the toolbar (without removing it) by calling
+ * #gtk_widget_hide_all.
  **/
 void 
 hildon_window_add_toolbar                       (HildonWindow *self, 
                                                  GtkToolbar *toolbar)
 {
     GtkBox *vbox;
-    HildonWindowPrivate *priv = HILDON_WINDOW_GET_PRIVATE (self);
+    HildonWindowPrivate *priv;
 
     g_return_if_fail (HILDON_IS_WINDOW (self));
     g_return_if_fail (toolbar && GTK_IS_TOOLBAR (toolbar));
-    g_assert (priv);
+
+    priv = HILDON_WINDOW_GET_PRIVATE (self);
 
     vbox = GTK_BOX (priv->vbox);
 
-    gtk_box_pack_start (vbox, GTK_WIDGET(toolbar), TRUE, TRUE, 0);
-    gtk_box_reorder_child (vbox, GTK_WIDGET(toolbar), 0);
+    gtk_box_pack_start (vbox, GTK_WIDGET (toolbar), TRUE, TRUE, 0);
+    gtk_box_reorder_child (vbox, GTK_WIDGET (toolbar), 0);
     gtk_widget_set_size_request (GTK_WIDGET (toolbar), -1, TOOLBAR_HEIGHT);
 
-    gtk_widget_queue_resize (GTK_WIDGET(self));
+    gtk_widget_queue_resize (GTK_WIDGET (self));
 }
 
 /**
@@ -1687,35 +1725,40 @@ hildon_window_add_toolbar                       (HildonWindow *self,
  * @self: A @HildonWindow
  * @toolbar: A #GtkToolbar to remove from the HildonWindow
  *
- * Removes a toolbar from the window.
+ * Removes a toolbar from the window. Note that this decreases the refference
+ * count on the widget. If you want to keep the toolbar alive call #g_object_ref 
+ * before calling this function.
  **/
 void
 hildon_window_remove_toolbar                    (HildonWindow *self, 
                                                  GtkToolbar *toolbar)
 {
-    HildonWindowPrivate *priv = HILDON_WINDOW_GET_PRIVATE (self);
-    
+    HildonWindowPrivate *priv;
+
     g_return_if_fail (HILDON_IS_WINDOW (self));
-    g_assert (priv);
     
-    gtk_container_remove (GTK_CONTAINER (priv->vbox), GTK_WIDGET(toolbar));
+    priv = HILDON_WINDOW_GET_PRIVATE (self);
+
+    gtk_container_remove (GTK_CONTAINER (priv->vbox), GTK_WIDGET (toolbar));
 }
 
 /**
  * hildon_window_get_menu:
  * @self : #HildonWindow
  * 
- * Gets the #GtMenu assigned to the #HildonAppview.
+ * Gets the #GtMenu assigned to the #HildonAppview. Note that the 
+ * window is still the owner of the menu.
  * 
- * Return value: The #GtkMenu assigned to this application view.
+ * Return value: The #GtkMenu assigned to this application view. 
  **/
 GtkMenu*
 hildon_window_get_menu                          (HildonWindow * self)
 {
-    HildonWindowPrivate *priv = HILDON_WINDOW_GET_PRIVATE (self);
-    
+    HildonWindowPrivate *priv;
+
     g_return_val_if_fail (HILDON_IS_WINDOW (self), NULL);
-    g_assert (priv);
+
+    priv = HILDON_WINDOW_GET_PRIVATE (self);
 
     return GTK_MENU (priv->menu);
 }
@@ -1728,24 +1771,28 @@ hildon_window_get_menu                          (HildonWindow * self)
  * Sets the menu to be used for this window. This menu overrides
  * a program-wide menu that may have been set with
  * hildon_program_set_common_menu. Pass NULL to remove the current
- * menu.
+ * menu. HildonWindow takes ownership of the passed menu and you're
+ * not supposed to free it yourself anymore.
  **/ 
 void
 hildon_window_set_menu                          (HildonWindow *self, 
                                                  GtkMenu *menu)
 {
-    HildonWindowPrivate *priv = HILDON_WINDOW_GET_PRIVATE (self);
+    HildonWindowPrivate *priv;
 
     g_return_if_fail (HILDON_IS_WINDOW (self));
-    g_assert (priv);
 
-    if (priv->menu != NULL) {
+    priv = HILDON_WINDOW_GET_PRIVATE (self);
+
+    if (priv->menu != NULL)
+    {
         gtk_menu_detach (GTK_MENU (priv->menu));
         g_object_unref (priv->menu);
     }
 
     priv->menu = (menu != NULL) ? GTK_WIDGET (menu) : NULL;
-    if (priv->menu != NULL) {
+    if (priv->menu != NULL)
+    {
         gtk_widget_set_name (priv->menu, "menu_force_with_corners");
         gtk_menu_attach_to_widget (GTK_MENU (priv->menu), GTK_WIDGET (self), &detach_menu_func);
         g_object_ref (GTK_MENU (priv->menu));
@@ -1763,11 +1810,11 @@ hildon_window_set_menu                          (HildonWindow *self,
 gboolean
 hildon_window_get_is_topmost                    (HildonWindow *self)
 {
-    HildonWindowPrivate *priv = HILDON_WINDOW_GET_PRIVATE (self);
+    HildonWindowPrivate *priv;
 
     g_return_val_if_fail (HILDON_IS_WINDOW (self), FALSE);
-    g_assert (priv);
 
+    priv = HILDON_WINDOW_GET_PRIVATE (self);
     return priv->is_topmost;
 }
 
