@@ -9,7 +9,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; version 2.1 of
- * the License.
+ * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +26,7 @@
 /**
  * SECTION:hildon-wizard-dialog
  * @short_description: A widget to create a guided installation
- * process wizard
+ * process wizard.
  *
  * #HildonWizardDialog is a widget to create a guided installation
  * process. The dialog has four standard buttons, previous, next,
@@ -427,13 +427,25 @@ response                                        (HildonWizardDialog *wizard_dial
 
     g_assert (priv);
 
+    current = gtk_notebook_current_page (notebook);
+
     switch (response_id) {
 
         case HILDON_WIZARD_DIALOG_PREVIOUS:
+            --current;
+            is_last = (current == last);
+            is_first = (current == 0);
+            make_buttons_sensitive (wizard_dialog,
+                                    !is_first, !is_first, !is_last); 
             gtk_notebook_prev_page (notebook); /* go to previous page */
             break;
 
         case HILDON_WIZARD_DIALOG_NEXT:
+            ++current;
+            is_last = (current == last);
+            is_first = (current == 0);
+            make_buttons_sensitive (wizard_dialog,
+                                    !is_first, !is_first, !is_last);
             gtk_notebook_next_page (notebook); /* go to next page */
             break;
 
